package com.benyun.core.dao;

import com.benyun.core.entity.Brand;
import com.benyun.core.entity.bo.BrandAddrDistribution;
import com.benyun.core.entity.bo.BrandSearch;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Mapper
public interface BrandMapper {
    List<Brand> searchList(String text);
    Brand searchById(String brandId);
    List<Brand> searchByIndustryCodeList(String brandId, String industryCode);
    List<BrandSearch> searchSearchByMulti(@Param("text") String text,@Param("industryCode") List<String> industryCode,@Param("countUp") Integer countUp,@Param("countDown") Integer countDown,@Param("coverUp") Integer coverUp,@Param("coverDown") Integer coverDown);
}
